/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2010 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/


/*
*****************************************************************************
*
* frequency analysis (power spectrum)
*
* Signal X sampled to N samples in X0, X1, X2, ... Xn-1
* Sample interval t(tau) seconds (i.e. 0.01 at 100 Hz samplefreq.)
*
* Powerspectrum P
* Calculated for N/2 frequencypoints op P0, P1, P2, ... Pn/2-1
* Frequencyresolution dF(delta F) = 1/Nt (i.e. 0.1 with 10 seconds signal (N = 1000))
*
* Calculate Pi, thus for frequency i * dF (i = 0, 1, 2, ... N/2-1)
*
* REi = t * {X0 * cos((2*pi*i/N) * 0)  +  X1 * cos((2*pi*i/N) * 1)  +  X2 * cos((2*pi*i/N) * 2)  +  ...  +  XN-1 * cos((2*pi*i/N) * (N-1))}
*
* IMi = t * {X0 * sin((2*pi*i/N) * 0)  +  X1 * sin((2*pi*i/N) * 1)  +  X2 * sin((2*pi*i/N) * 2)  +  ...  +  XN-1 * sin((2*pi*i/N) * (N-1))}
*
* Pi = (2/N*t) * {REi*REi + IMi*IMi}
*
*****************************************************************************
*/



#include "fourier.h"


#define _GNU_SOURCE


#define DUB_PI 6.28318530717958623199592693708837032318115234375



float fourier_transform_get_power_float(int kf, float *buf, int n, float samplefreq)
{
  int i;

  float tau,
        var,
        result,
        rei=0.0,
        imi=0.0;

  tau = 1.0 / samplefreq;

  var = (DUB_PI * (float)kf) / n;

  for(i=0; i<n; i++)
  {
    rei += (buf[i] * cos(var * i));

    imi += (buf[i] * sin(var * i));
  }

  rei *= tau;
  imi *= tau;

  result = (2.0 / (n * tau)) * ((rei * rei) + (imi * imi));

  if(!kf)    /* DC! */
  {
    return(result / 2.0);
  }

  return(result);
}


double fourier_transform_get_power_double(int kf, double *buf, int n, double samplefreq)
{
  int i;

  double tau,
         var,
         result,
         rei=0.0,
         imi=0.0;

  tau = 1.0 / samplefreq;

  var = (DUB_PI * (double)kf) / n;

  for(i=0; i<n; i++)
  {
    rei += (buf[i] * cos(var * i));

    imi += (buf[i] * sin(var * i));
  }

  rei *= tau;
  imi *= tau;

  result = (2.0 / (n * tau)) * ((rei * rei) + (imi * imi));

  if(!kf)    /* DC! */
  {
    return(result / 2.0);
  }

  return(result);
}

















